
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#ifdef _LINUX
#include <pwd.h>
#endif

#include "common.h"
#include "general_setting.h"
#include "msgbox.h"
#include "sflist.h"

GENERAL_SETTING general_setting;

static char *rc_file = NULL;
static char *rc_dir = NULL;
static char *vpo_default_file = NULL;
static char recent_regfile[1000];

typedef struct {
  int channel;
  int transpose;
  int loopbackdev;
} KB_INFO;

static KB_INFO div_info[KB_NUM];
static int inputdev = 0;


const char *general_setting_init()
{
  char path[1024];
  if (rc_file == NULL) {

#ifdef _LINUX
    struct passwd *p;
    uid_t u=getuid();
    p=getpwuid(u);
    sprintf(path,"%s/.vpo/",p->pw_dir);
#else
    sprintf(path,"./vpo/");
#endif
    
    rc_dir = strdup(path);
    
    if (access(rc_dir,X_OK)) {
#ifdef _LINUX
      if (mkdir(rc_dir , 0755)) {
#else
      if (mkdir(rc_dir)) {
#endif
	return "Couldn't create ~/.vpo/ ";
      }
    }

    sprintf(path, "%s.vporc", rc_dir);
    rc_file = strdup(path);

    sprintf(path, "%s.default.vpo", rc_dir);
    vpo_default_file = strdup(path);
  }
  return NULL;
}


void general_load_setting()
{
  char strtmp[10000];
  char section[10000];
  char value[10000];
  char key[10000];
  int i;
  if (rc_file==NULL) {
    general_setting_init();
    if (rc_file==NULL) {
      return;
    }
  }


  GetPrivateProfileString("Soundfont","num","0", strtmp, 16, rc_file);
  general_setting.sf_num = atoi(strtmp);
  for (i=0; i<general_setting.sf_num; i++) {
    sprintf(key, "sf%03d", i);
    GetPrivateProfileString("Soundfont", key, "\x22\x22,\x22\x22", value, 10000, rc_file);
    csv_read(value, general_setting.sf_name[i].name, 0);
    csv_read(value, general_setting.sf_name[i].path, 1);
  }
  
  GetPrivateProfileString("Timidity","location","/usr/local/bin/timidity_vpo",
			  general_setting.timi_location,255,rc_file);
  
  GetPrivateProfileString("Timidity","ctl_rate","1", strtmp,255,rc_file);
  general_setting.timi_ctlrate = atoi(strtmp);
  
  GetPrivateProfileString("Timidity","reverb","0", strtmp,255,rc_file);
  general_setting.timi_reverb = atoi(strtmp);
  
  GetPrivateProfileString("Timidity","chorus","0", strtmp,255,rc_file);
  general_setting.timi_chorus = atoi(strtmp);
  
  GetPrivateProfileString("Timidity","max_voices","384", strtmp,255,rc_file);
  general_setting.timi_voices = atoi(strtmp);
  
  GetPrivateProfileString("Timidity","options",
			  "-B4,8 -EO -Os -s44100 -EFresamp=l", 
			  general_setting.timi_options,255,rc_file);
  
  
  GetPrivateProfileString("EcaSound","location","/usr/local/bin/ecasound",
			  general_setting.eca_location,255,rc_file);
  
  GetPrivateProfileString("EcaSound","options",
			  "-i:stdin -b:17 -el:freeverb1,.750,.80,.2,.80", 
			  general_setting.eca_options,255,rc_file);
  
  GetPrivateProfileString("EcaSound","enable","0", strtmp,255,rc_file);
  general_setting.eca_enable = atoi(strtmp);
  
  for (i=0; i<KB_NUM; i++) {
    char default_loopback[8];
    sprintf(section, "Keyboard_%d", i+1);
    GetPrivateProfileString(section, "channel","1", value, 255,rc_file);
    div_info[i].channel = atoi(value)-1;
    GetPrivateProfileString(section, "transpose","0", value, 255,rc_file);
    div_info[i].transpose = atoi(value);
    sprintf(default_loopback, "%d", i+1);
    GetPrivateProfileString(section, "loopbackdev", default_loopback, value,255,rc_file);
    div_info[i].loopbackdev = atoi(value);
  }

  GetPrivateProfileString("General","recent_regfile",
			  rc_dir ? rc_dir:"", 
			  recent_regfile,999,rc_file);
  GetPrivateProfileString("General", "inputdev", "0", value,255,rc_file);
  inputdev = atoi(value);

}


static char* itoa2(int i)
{
  static char buf[100];
  sprintf(buf, "%d", i);
  return buf;
}


void general_save_setting()
{
  char section[10000];
  int i;

  if (rc_file==NULL) {
    general_setting_init();
    if (rc_file==NULL) {
      return;
    }
  }
 
  WritePrivateProfileString("Soundfont","num",itoa2(general_setting.sf_num), rc_file);
  for (i=0; i<general_setting.sf_num; i++) {
    char value[10000];
    sprintf(section, "sf%03d", i);
    sprintf(value,"\x22%s\x22,\x22%s\x22", general_setting.sf_name[i].name,general_setting.sf_name[i].path);
    WritePrivateProfileString("Soundfont", section, value, rc_file);
  }
 
  WritePrivateProfileString("General", "recent_regfile",
			    recent_regfile, rc_file);  
  
  WritePrivateProfileString("Timidity", "location",
			    general_setting.timi_location, rc_file);
  
  WritePrivateProfileString("Timidity", "ctl_rate",
			    itoa2(general_setting.timi_ctlrate), rc_file);
  
  WritePrivateProfileString("Timidity", "reverb",
			    itoa2(general_setting.timi_reverb), rc_file);
  
  WritePrivateProfileString("Timidity", "chorus",
			    itoa2(general_setting.timi_chorus), rc_file);
  
  WritePrivateProfileString("Timidity", "max_voices",
			    itoa2(general_setting.timi_voices), rc_file);

  WritePrivateProfileString("Timidity", "options",
			    general_setting.timi_options, rc_file);
  
  
  WritePrivateProfileString("EcaSound", "location",
			    general_setting.eca_location, rc_file);

  WritePrivateProfileString("EcaSound", "options",
			    general_setting.eca_options, rc_file);

  WritePrivateProfileString("EcaSound", "enable",
			    itoa2(general_setting.eca_enable), rc_file);


  for (i=0; i<KB_NUM; i++) {
    sprintf(section, "Keyboard_%d", i+1);
    WritePrivateProfileString(section, "channel",
			      itoa2(div_info[i].channel+1), rc_file);
    WritePrivateProfileString(section, "transpose",
			      itoa2(div_info[i].transpose), rc_file);
    WritePrivateProfileString(section, "loopbackdev",
                              itoa2(div_info[i].loopbackdev), rc_file);
  }

  WritePrivateProfileString("General", "inputdev",
			    itoa2(inputdev), rc_file);


}


char *general_get_default_vpofile()
{
  return vpo_default_file;
}


int general_get_div_channel(int div)
{
  return div_info[div].channel;
}


void general_set_div_channel(int div, int ch)
{
  if (div < 0 || div >= KB_NUM) {
    return;
  }
  div_info[div].channel = ch;
}

int general_get_div_transpose(int div)
{
  return div_info[div].transpose;
}


void general_set_div_transpose(int div, int tr)
{
  if (div < 0 || div >= KB_NUM) {
    return;
  }
  div_info[div].transpose = tr;
}

char *general_get_recent_registration_file()
{
  return recent_regfile;
}

void general_set_recent_registration_file(char *file)
{
  strcpy(recent_regfile, file);
}

int general_get_div_loopback_dev(int div)
{
  return div_info[div].loopbackdev;
}

void general_set_div_loopback_dev(int div, int dev)
{
  div_info[div].loopbackdev = dev;
}

int general_get_input_dev()
{
  return inputdev;
}

void general_set_input_dev(int dev)
{
  inputdev = dev;
}


char *general_get_timidity_location()
{
  return general_setting.timi_location;
}

void general_set_timidity_location(const char *location)
{
  strcpy(general_setting.timi_location, location);
}


void general_set_soundfont_name(char *sf_name, char *path)
{
  int i;
  int position = -1;
  GENERAL_SETTING* gs = &general_setting;

  for (i=0; i<gs->sf_num; i++) {
    if (strcmp(sf_name, gs->sf_name[i].name)==0) {
      position = i;
      break;
    }
  }
  if (position < 0) {
    position = gs->sf_num;
    gs->sf_num ++;
  }
  
  strcpy(gs->sf_name[position].name, sf_name);
  strcpy(gs->sf_name[position].path, path);
}


char *general_get_soundfont_path(char *sf_name, int flgAsk)
{
  int i;
  GENERAL_SETTING* gs = &general_setting;
  static char soundfont_fn[1024];

  for (i=0; i<gs->sf_num; i++) {
    if (strcmp(sf_name, gs->sf_name[i].name)==0) {
      return gs->sf_name[i].path;
    }
  }

  if (flgAsk) {
    char msg[4096];
    sprintf(msg,"Please specify the location of \n'%s'.", sf_name);
    if (msgbox(msg, "Select sound font.", "OK|Cancel",0)==0) {
      if (file_dialog(soundfont_fn, 
		      "Select SoundFont file", sflist_getSFFileName())==FALSE) {
	return NULL;
      }
      return soundfont_fn;
    }
  }

  return NULL;
}

const char *general_get_rc_dir()
{
	return rc_dir;
}

