/***************************************************
  Virtual Pipe Organ for Linux (init file handling)
  By Atsushi Ueda <a-ueda@japan-net.ne.jp>
****************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "common.h"
#include "ini.h"

#define KEYMAX 1000
#define STRMAX 1000
#define SECMAX 1000


typedef struct _NODE{
	char *section;
	char *key;
	char *string;
	struct _NODE* next;
} NODE;	

#define MAX(a,b)  (((a)>=(b)) ? (a) : (b))
#define MIN(a,b)  (((a)<=(b)) ? (a) : (b))

char* malloc_str(const char *str)
{
	char* res = (char*)malloc(strlen(str)+1);
	if (res) {
		strcpy(res, str);
	}
	return res;
}


void spccut(char *str)
{
	int len;

	len = strlen(str);

}

BOOL eq_sep(char* str, char** _left, char** _right)
{
	static char left[KEYMAX], right[STRMAX];
	char *p, *pw;
	int len;

	*_left = *_right = NULL;

	pw = left;

	*pw = 0;
	for (p=str,len=0; *p && (*p!='=') && len<KEYMAX-2; p++,len++)
		*(pw++) = *p;
	*pw = 0;

	if (*p != '=') return FALSE;

	pw = right;
	*pw = 0;
	for (p++,len=0; *p && len<STRMAX-2 ; p++,len++) 
		*(pw++) = *p;
	*pw = 0;

	*_left = left;
	*_right = right;

	//printf("[%s] [%s]\n", left,right);

	return TRUE;
}


static NODE* startp;


BOOL read_ini(const char *fn)
{
	char section[SECMAX+1];
	FILE *fp;
	NODE *cur_ptr = NULL, *tmp;
	char *left, *right;
	struct stat statbuf;
	startp = NULL;

	stat(fn, &statbuf);
	if (statbuf.st_mode & S_IFDIR) {
	  return FALSE;
	}

	if (!(fp = fopen(fn, "r"))) return FALSE;
	strcpy(section, "NoSection");

	for (;;) {
		char *p;
		char buf[2010];
		fgets(buf, 2000, fp);
		if (feof(fp)) break;
		if ((p=strchr(buf,0x0d))) *p=0;
		if ((p=strchr(buf,0x0a))) *p=0;

		if (buf[0] == '[') {
			strncpy(section , buf+1, MIN(strlen(buf), SECMAX));
			section[SECMAX]=0;
			if ((p=strchr(section, ']'))) {
				*p = 0;
			}
			continue;
		}

		if (eq_sep(buf, &left, &right)==FALSE) {
			continue;
		}

		if (!(tmp = (NODE*)malloc(sizeof(NODE)))) {
		  goto owari;
		}

		tmp->section = tmp->key = tmp->string = NULL;
		tmp->next = NULL;
		if (!(tmp->section = malloc_str(section))) goto owari;
		if (!(tmp->key = malloc_str(left))) goto owari;
		if (!(tmp->string = malloc_str(right))) goto owari;
		if (cur_ptr) {
			cur_ptr->next = tmp;
			cur_ptr = tmp;
		} else {
			cur_ptr = tmp;
			startp = tmp;
		}
	}

	fclose(fp);
	return TRUE;
owari:
	fclose(fp);
	return FALSE;
}

void removeall()
{
	NODE *p, *next;
	p = startp;
	while (p) {
		next = p->next;
		if (p->key) {
			free(p->key);
		}
		if (p->section) {
			free(p->section);
		}
		if (p->string) {
			free(p->string);
		}
		free(p);
		p = next;
	}
}



NODE* search_node(const char *section, const char *key)
{
	NODE* p;
	for (p = startp; p; p=p->next) {
		if (strcmp(p->section, section)==0 && strcmp(p->key, key)==0) {
			return p;
		}
	}
	return NULL;
}


BOOL GetPrivateProfileString(const char* section, const char* key, const char *defaultstr, char *res, int maxlen, const char *fn)
{
	if (read_ini(fn) == TRUE) {
		NODE *node;
		if ((node = search_node(section, key))) {
			strncpy(res, node->string, maxlen);
		}
		removeall();
		if (node) {
			return TRUE;
		}
	}
	strncpy(res, defaultstr, maxlen);
	return TRUE;
}



BOOL WritePrivateProfileString(const char* section, const char* key, const char* string, const char* fn)
{
	NODE *tmp, *p;
	FILE *fp;
	char prev_section[SECMAX+1];

	if (!read_ini(fn)) {
	  //return FALSE;
	}

	if ((p = search_node(section, key))) {
		free(p->string);
		p->string=NULL;
		if (!(p->string = malloc_str(string))) {
			removeall();
			return FALSE;
		}
	} else {
		if (!(tmp = (NODE*)malloc(sizeof(NODE)))) {
			removeall();
			return FALSE;
		}
		tmp->section = malloc_str(section);
		tmp->key = malloc_str(key);
		tmp->string = malloc_str(string);
		tmp->next = NULL;
		if (!startp) {
			startp = tmp;
		} else {
			for (p = startp; p->next; p=p->next) {
				if (strcmp(p->section, section)==0) {
					for (;;) {
						if (p->next==NULL) break;
						if (strcmp(p->next->section,section)) break;
						p=p->next;
					}
					break;
				}
			}
			if (p->next) {
				tmp->next = p->next;
			}
			p->next=tmp;
		}
	}

	if (!(fp=fopen(fn,"w"))) {
		removeall();
		return FALSE;
	}

	strcpy(prev_section,"");
	for (tmp=startp;tmp;tmp=tmp->next) {
		if (strcmp(prev_section, tmp->section)) {
			fprintf(fp,"[%s]\n", tmp->section);
			strcpy(prev_section, tmp->section);
		}
		fprintf(fp,"%s=%s\n", tmp->key, tmp->string);
	}

	fclose(fp);

	removeall();
	return TRUE;
}



