/**********************************************
  Virtual Pipe Organ for Linux 
  (Reading sound name list from Soundfont file)
  By Atsushi Ueda <a-ueda@japan-net.ne.jp>
***********************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "sflist.h"

#pragma pack(1)

typedef struct {
  char name[20];
  int16_t preset;
  int16_t bank;
  int16_t a;
  int32_t b,c,d;
} sfPresentHeader;

unsigned char name[4096];
unsigned char filename[256];


static int cmp(const void *p01, const void *p02)
{
  sfPresentHeader* p1 = (sfPresentHeader*)p01;
  sfPresentHeader* p2 = (sfPresentHeader*)p02;
  int res;
  if ((res = p1->bank - p2->bank)) return res;
  return p1->preset - p2->preset;
}


unsigned long getud(unsigned char *p)
{
  return (unsigned long)p[3] << 24 | 
         (unsigned long)p[2] << 16 | 
         (unsigned long)p[1] << 8  | 
         (unsigned long)p[0];
}



/* number of data */
static int num = 0;

/* preset name data list */
static sfPresentHeader *data = NULL;


int sflist_getnum()
{
  return num;
}

char* sflist_getname(int bank, int preset)
{
  static char buf[21];
  sfPresentHeader tmp, *res;

  if (!data) return NULL;
  
  tmp.bank = bank;
  tmp.preset = preset;

  res = (sfPresentHeader*)bsearch(&tmp, data, num, sizeof(sfPresentHeader), cmp);
  if (!res) return NULL;
  memcpy(buf, res->name, 20);
  buf[20] = 0;
  return buf;
}

char *sflist_getSFName()
{
  return (char*)name;
}

char *sflist_getSFFileName()
{
  return (char*)filename;
}


char* sflist_getname_n(int data_no, int *bank, int *preset)
{
  if (!data) return NULL;
  if (data_no >= num) return NULL;
  *bank = data[data_no].bank;
  *preset = data[data_no].preset;
  return data[data_no].name;
}

int sflist_load(char *fn)
{
  int err=-1;
  int i;
  unsigned char size[4];
  unsigned char ckID[6];
  int info_list_size;
  int info_list_read=0;
  FILE *fp;

  name[0] = 0;
  filename[0]=0;

  printf("<%s>\n",fn);

  fp = fopen(fn,"rb");
  if (!fp) goto owari;
  strcpy((char*)filename, fn);
  fread(ckID,1,4,fp);
  if (memcmp(ckID,"RIFF",4)) {err=0; goto owari;}
  fread(size,1,4,fp);
  fread(ckID,1,4,fp);
  if (memcmp(ckID,"sfbk",4)) {err=1; goto owari;}
  fread(ckID,1,4,fp);
  if (memcmp(ckID,"LIST",4)) {err=2; goto owari;}
  fread(size,1,4,fp);
  info_list_size = getud(size);
//printf("info size = %d\n",getud(size));
  fread(ckID,1,4,fp);

  if (memcmp(ckID,"INFO",4)) {err=3; goto owari;}
  info_list_read += 4;
  ckID[5] = 0;
  for (;;) {
    fread(ckID,1,4,fp);
    ckID[5] = 0;
    if (memcmp(ckID,"LIST",4)==0) break;
    fread(size,1,4,fp);
    if (memcmp(ckID,"INAM",4)==0) {
      fread(name, 1, getud(size), fp);
      name[getud(size)] = 0;
      printf("Name = \x22%s\x22\n", name);
    } else {
      fseek(fp, getud(size), SEEK_CUR);
    }
    info_list_read += 4;
    info_list_read += getud(size);
    //printf("   ckID=%s  size=%d  read=%d\n", ckID, getud(size), info_list_read);
    //if (info_list_read >= info_list_size) break;
  }

  // (λǼΡLISTɤ߹ߺ(sdta)) 
  if (memcmp(ckID,"LIST",4)) {err=5; goto owari;}
  fread(size,1,4,fp);
  fseek(fp, getud(size), SEEK_CUR);
  fread(ckID,1,4,fp);
  if (memcmp(ckID,"LIST",4)) {err=6; goto owari;}
  fread(size,1,4,fp);
  fread(ckID,1,4,fp);
  if (memcmp(ckID,"pdta",4)) {err=7; goto owari;}
  fread(ckID,1,4,fp);
  if (memcmp(ckID,"phdr",4)) {err=8; goto owari;}
  fread(size,1,4,fp);
  //printf("%d, %d\n", getud(size), sizeof(sfPresentHeader));
  num = getud(size) / sizeof(sfPresentHeader);

  if (data) {
    free(data);
  }
  data = (sfPresentHeader*)malloc(sizeof(sfPresentHeader)*num);


  for (i=0; i<num; i++) {
    sfPresentHeader tmp;
    fread(&tmp, sizeof(sfPresentHeader), 1, fp);
    if (strcmp(tmp.name,"EOP")!=0) {
      data[i] = tmp;
    } else {
      i--;
      num--;
    }
  }

  qsort(data, num, sizeof(sfPresentHeader), cmp);
  //num--; // ignore EOP record

  fclose(fp);
  return TRUE;

owari: 
  if (fp) fclose(fp);
  num = 0;
  printf("error %d  [%s]\n",err,ckID); 
  return FALSE;
}


int sflist_make_timidity_config_file(const char *sf_fn, const char *out_fn)
{
	FILE *fp = fopen(out_fn, "w");
	if (!fp) {
		return FALSE;
	}

	int prev_bank = -1;

	fprintf(fp, "soundfont %s order=0\n", sf_fn);

	for (int i=0; i<num; i++) {
		if (prev_bank != data[i].bank) {
			prev_bank = data[i].bank;
			if (data[i].bank > 127) {
				break;
			}
			fprintf(fp, "\nbank %d\n", data[i].bank);
		}
		fprintf(fp, "%d %%font %s %d %d\n", 
			data[i].preset,
			sf_fn,
			data[i].bank,
			data[i].preset);
	}

	fclose(fp);

	return TRUE;

}


